@@ -3,6 +3,7 @@ package KVM::Kavoom::Instance;
 use re '/aa';
 
 use Class::Clarity -self;
+
 use IO::File;
 use IO::Socket::UNIX;
 use File::Copy qw(copy);
@@ -16,10 +17,17 @@ field name;
 sub configvar(*@) {
     my $name = shift;
 	if(@_) {
-		my $postproc = shift;
-		field($name, eval qq{sub { my \$self = shift; return \$postproc->(\$self->config->$name(@_)) }});
+		my ($postproc, @args) = @_;
+		my $isset = "${name}_isset";
+		field($name, sub {
+			my $self = shift;
+			my $config = $self->config;
+			return $config->$isset
+				? $config->$name
+				: $postproc->($self, @args);
+		});
 	} else {
-		field($name, eval qq{sub { my \$self = shift; return \$self->config->$name(@_) }});
+		field($name, sub { shift->config->$name });
 	}
 }
 
@@ -36,12 +44,24 @@ configvar cache;
 configvar aio;
 configvar serialport;
 configvar kvm;
+configvar usb;
 configvar vnc;
 configvar mem;
 configvar cpus;
 configvar acpi;
 configvar platform;
 configvar ovmfdir;
+configvar chipset => sub {
+	my $platform = $self->platform;
+	return undef unless defined $platform;
+	return undef if $platform eq 'bios';
+	return 'q35';
+};
+configvar tablet => sub {
+	return exists $self->{tablet}
+		? $self->{tablet}
+		: $self->{vnc};
+};
 configvar hugepages => sub {
 	my $hugepages;
 
@@ -128,13 +148,6 @@ sub id {
 
 field vhost_net => sub { -e '/dev/vhost-net' };
 
-sub tablet {
-	return $self->{tablet} = bool(shift) if @_;
-	return exists $self->{tablet}
-		? $self->{tablet}
-		: $self->{vnc};
-}
-
 sub nictype {
 	return $self->virtio ? 'virtio-net' : 'e1000';
 }
@@ -168,6 +181,7 @@ sub command {
 		-readconfig => $devices,
 		-daemonize,
 		-nodefaults,
+		-nographic,
 		-vga => 'cirrus',
 	);
 
@@ -294,14 +308,21 @@ sub devices_write {
 	$self->devices_stanza($fh, memory => undef, size => $mem)
 		if defined $mem;
 
+	my $chipset = $self->chipset;
+	my @chipset = (type => $chipset) if defined $chipset;
+	my @usb = (usb => 'on') if $self->usb;
+	if(my @machine = (@chipset, @usb)) {
+		$self->devices_stanza($fh, machine => undef, @machine);
+	}
+
 	$self->devices_stanza($fh, chardev => 'monitor',
 		backend => 'socket',
 		server => 'on',
 		wait => 'off',
 		path => "$rundir/$name.monitor",
 	);
-	
-	$self->devices_stanza($fh, mon => 'monitor', 
+
+	$self->devices_stanza($fh, mon => 'monitor',
 		mode => 'readline',
 		chardev => 'monitor',
 	);
@@ -312,8 +333,8 @@ sub devices_write {
 		wait => 'off',
 		path => "$rundir/$name.qmp",
 	);
-	
-	$self->devices_stanza($fh, mon => 'qmp', 
+
+	$self->devices_stanza($fh, mon => 'qmp',
 		mode => 'control',
 		chardev => 'qmp',
 		pretty => 'on',
@@ -321,7 +342,7 @@ sub devices_write {
 
 	if($self->vnc) {
 		my $id = $self->id;
-		$self->devices_stanza(vnc => 'default',
+		$self->devices_stanza($fh, vnc => 'default',
 			vnc => "localhost:$id",
 		);
 	}
@@ -334,7 +355,7 @@ sub devices_write {
 				wait => 'off',
 				path => "$rundir/$name.serial-$i",
 			);
-			$self->devices_stanza($fh, device => "serial-$i", 
+			$self->devices_stanza($fh, device => "serial-$i",
 				driver => 'isa-serial',
 				chardev => "serial-$i",
 			);
@@ -353,7 +374,7 @@ sub devices_write {
 				wait => 'off',
 				path => "$rundir/$name.console-$i",
 			);
-			$self->devices_stanza($fh, device => 'virtconsole', 
+			$self->devices_stanza($fh, device => 'virtconsole',
 				driver => 'virtconsole',
 				chardev => "console-$i",
 				name => "hvc$i",
@@ -361,52 +382,6 @@ sub devices_write {
 		}
 	}
 
-	my $nics = $self->nics;
-	my $nictype = $self->nictype;
-	my @vhost_net = (vhost => 'on')
-		if $self->vhost_net;
-	while(my ($i, $mac) = each @$nics) {
-		$self->devices_stanza($fh, netdev => "net-$i",
-			type => 'tap',
-			ifname => $name.$i,
-			@vhost_net,
-		);
-		$self->devices_stanza($fh, device => "net-$i", 
-			driver => $nictype,
-			netdev => "net-$i",
-			mac => $mac,
-		);
-	}
-
-	my $disks = $self->disks;
-	my $disktype = $self->disktype;
-	while(my ($i, $disk) = each @$disks) {
-		my $cache = $self->cache;
-		my $aio = $self->aio;
-		my %opt;
-		die "no such file or directory: $disk\n"
-			unless -e $disk;
-		if(-b _) {
-			$cache //= 'none';
-			$aio //= 'native' if $cache eq 'none';
-			$opt{format} = 'raw';
-		}
-		my $serial = substr(md5_base64($disk), 0, 20);
-		$serial =~ tr{+/}{XY};
-		$opt{serial} = $serial;
-		$opt{cache} = $cache if defined $cache;
-		$opt{aio} = $aio if defined $aio;
-		$self->devices_stanza($fh, drive => "blk-$i",
-			file => $disk,
-			if => 'none',
-			%opt,
-		);
-		$self->devices_stanza($fh, device => "blk-$i", 
-			driver => $disktype,
-			drive => "blk-$i",
-		);
-	}
-
 	my $platform = $self->platform;
 	if($platform eq 'efi') {
 		$self->lock;
@@ -440,6 +415,52 @@ sub devices_write {
 		);
 	}
 
+	my $disks = $self->disks;
+	my $disktype = $self->disktype;
+	while(my ($i, $disk) = each @$disks) {
+		my $cache = $self->cache;
+		my $aio = $self->aio;
+		my %opt;
+		die "no such file or directory: $disk\n"
+			unless -e $disk;
+		if(-b _) {
+			$cache //= 'none';
+			$aio //= 'native' if $cache eq 'none';
+			$opt{format} = 'raw';
+		}
+		my $serial = substr(md5_base64($disk), 0, 20);
+		$serial =~ tr{+/}{XY};
+		$opt{serial} = $serial;
+		$opt{cache} = $cache if defined $cache;
+		$opt{aio} = $aio if defined $aio;
+		$self->devices_stanza($fh, drive => "blk-$i",
+			file => $disk,
+			if => 'none',
+			%opt,
+		);
+		$self->devices_stanza($fh, device => "blk-$i",
+			driver => $disktype,
+			drive => "blk-$i",
+		);
+	}
+
+	my $nics = $self->nics;
+	my $nictype = $self->nictype;
+	my @vhost_net = (vhost => 'on')
+		if $self->vhost_net;
+	while(my ($i, $mac) = each @$nics) {
+		$self->devices_stanza($fh, netdev => "net-$i",
+			type => 'tap',
+			ifname => $name.$i,
+			@vhost_net,
+		);
+		$self->devices_stanza($fh, device => "net-$i",
+			driver => $nictype,
+			netdev => "net-$i",
+			mac => $mac,
+		);
+	}
+
 #	$self->devices_stanza($fh, drive => 'cdrom',
 #		if => 'none',
 #	);
